@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005 - 2007 Miek Gieben
+ * Copyright (c) 2005 - 2009 Miek Gieben
  * See LICENSE for the license
  *
  * Directory crawler
@@ -12,15 +12,17 @@ extern time_t opt_timestamp;
 extern gint opt_verbose;
 extern GSList *pregex_list;
 
+#if 0
 /* xattr.c */
 uid_t read_attr_uid(char *path, uid_t u);
 gid_t read_attr_gid(char *path, gid_t g);
+#endif
 
-static struct entry *
-entry_dup(struct entry *f)
+static struct r_entry *
+entry_dup(struct r_entry *f)
 {
-        struct entry *g;
-        g = g_malloc(sizeof(struct entry));
+        struct r_entry *g;
+        g = g_malloc(sizeof(struct r_entry));
 
         g->f_name       = g_strdup(f->f_name);
         g->f_name_size  = f->f_name_size;
@@ -37,7 +39,7 @@ entry_dup(struct entry *f)
 }
 
 static void
-entry_free(struct entry *f)
+entry_free(struct r_entry *f)
 {
 	g_free(f->f_name);
 	g_free(f);
@@ -54,7 +56,7 @@ dir_prepend(GTree *t, char *path)
 	char *path2;
 	size_t len;
 	struct stat s;
-	struct entry e;
+	struct r_entry e;
 
 	path2 = g_strdup(path);
 	len   = strlen(path);
@@ -82,6 +84,12 @@ dir_prepend(GTree *t, char *path)
 		e.f_dev       = s.st_dev;
 		e.f_rdev      = s.st_rdev;
 		e.f_ino       = s.st_ino;
+		e.f_lnk	      = 0;
+
+		/* symlinks; also put the -> name in f_name */
+		if (S_ISLNK(s.st_mode))
+			e = *(sym_link(&e, NULL));
+
 		g_tree_insert(t, (gpointer) entry_dup(&e), VALUE);
 		*c = DIR_SEP;
 		p = c++;
@@ -95,11 +103,11 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 {
 	DIR 		*dir;
 	struct dirent 	*dent;
-	struct entry    *directory;
+	struct r_entry  *directory;
 	char 		*curpath;
 	gchar		*lnk;
 	struct stat   	s;
-	struct entry	pop;
+	struct r_entry  pop;
 	struct remove_path rp;
 	dev_t 		current_dev;
 	size_t 		curpath_len;
@@ -107,8 +115,8 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 	/* dir stack */
 	gint32 d = 0;
 	gint32 dstack_cnt  = 1;
-	struct entry **dirstack =
-		g_malloc(dstack_cnt * D_STACKSIZE * sizeof(struct entry *));
+	struct r_entry **dirstack =
+		g_malloc(dstack_cnt * D_STACKSIZE * sizeof(struct r_entry *));
 
 	if (!(dir = opendir(path))) {
 		/* non-dirs are also allowed, check for this, if it isn't give the error */
@@ -136,8 +144,8 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 	current_dev = s.st_dev;
 
 	while((dent = readdir(dir))) {
-		if (!g_ascii_strcasecmp(dent->d_name, ".") || 
-				!g_ascii_strcasecmp(dent->d_name, ".."))
+		if (!strcmp(dent->d_name, ".") || 
+				!strcmp(dent->d_name, ".."))
 			continue;
 
 		if (strcmp(path, DIR_SEP_STR) == 0)  {
@@ -200,28 +208,13 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 
 			/* hardlinks */
 			if (s.st_nlink > 1) {
-				if (( lnk = hardlink(linkhash, &pop))) {
-					/* we got a match back */
-					pop.f_size = strlen(pop.f_name);  /* old name length */
-					lnk = g_strdup_printf("%s -> %s", pop.f_name, lnk);
-					pop.f_lnk = 1;
-					pop.f_name = lnk;
-					pop.f_name_size = strlen(pop.f_name);
-				}
-			}
-			/* symlinks; also put the -> name in f_name */
-			if (S_ISLNK(s.st_mode)) {
-				char buf[BUFSIZE + 1]; 
-				ssize_t i;
-				if ((i = readlink(pop.f_name, buf, BUFSIZE)) == -1) {
-					msg(_("Error reading link `%s\': %s"), pop.f_name, strerror(errno));
-				} else {
-					buf[i] = '\0';
-					pop.f_size = strlen(pop.f_name); /* old name length */
-					pop.f_name = g_strdup_printf("%s -> %s", pop.f_name, buf);
-					pop.f_name_size = strlen(pop.f_name);
+				if ((lnk = hard_link(linkhash, &pop))) {
+					/* we got a match back, modify pop */
+					pop = *(sym_link(&pop, lnk));
 				}
 			}
+			if (S_ISLNK(s.st_mode)) 
+				pop = *(sym_link(&pop, NULL));
 
 			g_tree_insert(t, (gpointer) entry_dup(&pop), VALUE);
 			g_free(curpath);
@@ -239,7 +232,7 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 				continue;
 			}
 
-			dirstack[d] = g_malloc(sizeof(struct entry));
+			dirstack[d] = g_malloc(sizeof(struct r_entry));
 			dirstack[d]->f_name       = g_strdup(curpath); 
 			dirstack[d]->f_name_size  = curpath_len;
 			dirstack[d]->f_uid	  = s.st_uid;
@@ -255,7 +248,7 @@ dir_crawl(GTree *t, GHashTable *linkhash, char *path)
 			if (d++ % D_STACKSIZE == 0) {
 				dirstack = g_realloc(dirstack, 
 						++dstack_cnt * D_STACKSIZE * 
-						sizeof(struct entry *));
+						sizeof(struct r_entry *));
 			}
 			g_free(curpath);
 			continue;
